



%    Stock and Watson (1988) "Testing for common trends" Journal of
%    American Statistical Association. Vol.83, 1097-1107
%    Bai, J., Ng, S., 2004. A PANIC attack on unit roots
%    and cointegration. Econometrica 72, 1127-1177.



% tmpData is observed

clear all;
clc;
tic

[T,N]=size(tmpData);  % tmpData is observed data
[V,D]=eig(tmpData'*tmpData);

mse=zeros(N,3);
Sigma=zeros(N,1);
BIC3=zeros(N,1);




%  k=5;    % number of factor
 k=3;       % number of factor
error=zeros(T,N);
Ft=zeros(T,k);
factor=tmpData*V(:,1:k)*sqrt(N)./N;
residual=tmpData-factor*V(:,1:k)';
ac=zeros(k,8);
df=zeros(k,1);
dfval=zeros(k,1);

% sample autocorrelation and augument ADF test
for i=1:k
%    ac(i,:)=sacf(factor(:,i),8);
%     ac(i,:)=autocoeff(factor(:,i),8);
   [df(i), dfval(i)]=augdf(factor(:,i),2,4);
end 

stat=zeros(1,N);
pval=zeros(1,N);
cval=zeros(1,N);
resid=zeros(1,N);
lags=zeros(1,N);
Fstat=zeros(1,k);
Fpval=zeros(1,k);
cumResid=zeros(T-1,N);
maxlags=12;

diffFactor=diff(factor);   % dimension (T-1)*N
diffResid=diff(residual);  % dimension (T-1)*N

for i=1:N
    for t=2:(T-1)
        cumResid(t,i)=sum(diffResid(2:t,i));            
    end
    [stat(i),pval(i)]=augdf(cumResid(:,i),0,6);    % augment Dickey-Fuller test
     
end


for i=1:k
    for t=1:T
       Ft(t,i)=sum(factor(1:t,i));
      
    end
     [Fstat(i),Fpval(i)]=augdf(Ft(:,i),1,6);
end
    
% Pooled test for idiosyncratic errors  P1140 in Bai and Ng (2004)

PooledADF=(-2*sum(log(pval))-2*N)/sqrt(4*N);  % N is infinite
Poolpvalue=2*(1-normcdf(PooledADF));
% PooledADF=-2*sum(log(pval));   % N is finite
% chiADF=1-chi2cdf(PooledADF,4);

% demean factors
deFactor=factor-ones(T,1)*mean(factor);
[B,BValue]=eig(deFactor'*deFactor);

% Qf and Modified Qf test
p=1;
MQf=zeros(k,1);
lamda=zeros(k,1);
   
errorY=zeros(T,k);
Y=deFactor*B(:,1:k);
for i=1:k
    [parAR,LL,errorY(:,i)]=armaxfilter(Y(:,i),0,1,0);
    %[parAR,LL,errorY(:,i)]=armaxfilter(deFactor(:,i),0,1,0);
end

% Bai and Ng (2004)
%     covMetric=0.5*sum(errorY((2+p):T,:)'*errorY((1+p):(T-1),:)+errorY((1+p):(T-1),:)'*errorY((2+p):T,:));
% Stock and Watson (1988)
    covMetric=sum(errorY((2+p):T,:)'*errorY((1+p):(T-1),:));
varMetric=sum(errorY((1+p):(T-1),:)'*errorY((1+p):(T-1),:));
autoMetric=covMetric./varMetric;
[autV,autD]=eig(autoMetric'*autoMetric);
[x1,y1]=size(autD);
lamda(1:x1,:)=sort(abs(autD)*ones(x1,1),'descend');

MQf(:,1)=T*(lamda-1);

% Qc and Modified Qc test 
MQc=zeros(k,1);
lamdaY=zeros(k,1);
% (Stock & Watson)
MM=(varMetric./T+0.5*covMetric./T)./T^2;
covY=sum(Y((2+p):T,:)'*Y((1+p):(T-1),:))./T^2;
varY=sum(Y((1+p):(T-1),:)'*Y((1+p):(T-1),:))./T^2;

% (Bai and Ng)

% MM=0.5*((varMetric./T+0.5*covMetric./T)+(varMetric./T+0.5*sum(errorY((1+p):(T-1),:)'*errorY((2+p):T,:))))./T^2;
% covY=0.5*sum(Y((2+p):T,:)'*Y((1+p):(T-1),:)+Y((1+p):(T-1),:)'*Y((2+p):T,:))./T^2;
% varY=sum(Y((1+p):(T-1),:)'*Y((1+p):(T-1),:))./T^2;

autoY=(covY-MM)./varY;
[autYV,autYD]=eig(autoY'*autoY);   
[x1,y1]=size(autYD);
lamdaY(1:x1,:)=sort(autYD*ones(x1,1),'descend');
MQc(:,1)=T*(lamdaY-1);


























